@@ -13,18 +13,19 @@ names are all lowercase letters.
 '''
 
 from morpholib.tools.basics import tau, argShift, argShiftArray, arcCenter, arcCenterArray
+import math
 import numpy as np
 import cairo
 cr = cairo
-from warnings import filterwarnings
+# from warnings import filterwarnings
 
 # Ignore warnings by default.
 # This is mainly for the sake of the Spline class, which
 # uses some non-standard arithmetic intentionally.
-filterwarnings("ignore")
+# filterwarnings("ignore")
 
-version = "0.3.0"  # Current public morpho version
-internalVersion = "2.2.6"  # Current internal morpho version
+version = "0.5.0"  # Current public morpho version
+internalVersion = "2.3.1"  # Current internal morpho version
 subversion = ""
 DEBUG_MODE = False
 
@@ -92,14 +93,17 @@ class Tweenable(object):
         # Convert tags to proper format.
         if tags is None:
             tags = set()
-        elif type(tags) is set:
+        # elif type(tags) is set:
+        elif isinstance(tags, set):
             # tags = tags.copy()
             pass
-        elif type(tags) is str:
+        # elif type(tags) is str:
+        elif isinstance(tags, str):
             tags = {tags,}
-        elif type(tags) in (list, tuple):
+        # elif type(tags) in (list, tuple):
+        elif isinstance(tags, list) or isinstance(tags, tuple):
             tags = set(tags)
-        elif type(tags) is not set:
+        else:
             raise TypeError("Given tags attribute is not a valid type of set, None, str, list, or tuple.")
 
         self.name = name
@@ -132,12 +136,14 @@ class Tweenable(object):
             value=self.value,
             metadata=self.metadata)
 
-        # Attempt to make a copy of the value if the copy() method exists.
-        if "copy" in dir(self.value):
-            try:
-                twCopy.value = self.value.copy()
-            except Exception:  # Upon failure, just reassign and hope for the best.
-                twCopy.value = self.value  # Not sure this line is needed
+        try:
+            twCopy.value = self.value.copy()
+        except Exception:  # Upon failure, just reassign and hope for the best.
+            # Actually, I think I DO want this line. It looks redundant,
+            # but if the try clause somehow messed up the assignment
+            # process itself, I'd like to reassign the initial value.
+            twCopy.value = self.value
+
         return twCopy
 
     def __repr__(self):
@@ -336,8 +342,29 @@ def screenCoords(z, view, ctx):
 # Alternate name for screenCoords()
 pixelCoords = screenCoords
 
+
+# Calls ctx.save() and then creates a context manager object for the
+# given ctx object that can be used in a `with` statement like this:
+#   with SavePoint(ctx):
+#       ...
+# and ctx.restore() will be called at the end of the block
+class SavePoint(object):
+    def __init__(self, ctx):
+        self.ctx = ctx
+        # print("ctx saved!")
+        self.ctx.save()
+
+    def __enter__(self):
+        return self
+
+    def __exit__(self, type, value, traceback):
+        self.ctx.restore()
+        # print("ctx restored!")
+
+# pushPhysicalCoords(view, ctx)
+#
 # Starting from a cairo coordinate system in screen coordinates
-# (which is default after calling setupContext()), changes the
+# (which is default after calling setupContext()), changes the cairo
 # coordinate system temporarily into physical coordinates based
 # on the viewbox you provide.
 #
@@ -348,6 +375,12 @@ pixelCoords = screenCoords
 # effects of pushPhysicalCoords()! Forgetting to will undoubtedly lead to all
 # kinds of trouble!
 #
+# To automatically handle calling ctx.restore(), you can call this
+# function as part of a `with` statement:
+#   with morpho.pushPhysicalCoords(view, ctx):
+#       ...
+# At the end of the block, ctx.restore() will be called automatically.
+#
 # To use it correctly, you should call this function BEFORE any
 # calls that modify the context's CTM as part of drawing your given figure.
 # The idea is that you apply all of your CTM modifications pretending the
@@ -364,6 +397,19 @@ def pushPhysicalCoords(view, ctx):
     WIDTH = surface.get_width()
     HEIGHT = surface.get_height()
 
+    savept = SavePoint(ctx)
+    ctx.scale(WIDTH/(b-a), HEIGHT/(d-c))
+    ctx.translate(-a, -c)
+
+    return savept
+
+def pushPhysicalCoords_old(view, ctx):
+    a,b,c,d = view
+
+    surface = ctx.get_target()
+    WIDTH = surface.get_width()
+    HEIGHT = surface.get_height()
+
     ctx.save()
     ctx.scale(WIDTH/(b-a), HEIGHT/(d-c))
     ctx.translate(-a, -c)
@@ -455,6 +501,46 @@ def pixelAspectRatioWH(view, ctx):
 
 I2 = np.eye(2)
 
+# Returns the special transformation matrix that compensates
+# for rotations and transformations in a non-square view.
+#
+# INPUTS
+# par = Pixel aspect ratio as computed by
+#       morpho.pixelAspectRatioWH()
+# rotation = Rotation in radians. Default: 0.
+# transform = Local transformation matrix. Default: I2
+# inverse (keyword-only) = Boolean indicating whether the
+#           inverse transformation should be returned.
+#           Default: False
+#
+# More precisely, if S represents the matrix that locally
+# transforms pixel coords to physical coords
+# according to the par, this function returns
+# S @ R @ T @ S.inv
+# where R and T are the rotation and transform matrices.
+def parconj(par, rotation=0, transform=I2, *, inverse=False):
+    # Construct 2D rotation matrix
+    if rotation == 0:
+        R = I2
+    else:
+        c = math.cos(rotation)
+        s = math.sin(rotation)
+        R = np.array([[c, -s],[s, c]], dtype=float)
+
+    # S represents the linear transformation that converts
+    # pixel coordinates to physical coordinates.
+    # Since S is diagonal, conjugating with it is easy, just
+    # multiply element-wise by parmat:
+    # S.M.S^-1 = parmat*M
+    parmat = np.array([[1, 1/par],[par, 1]], dtype=float)
+    if inverse:
+        if np.array_equal(transform, I2):
+            return parmat*R.T
+        else:
+            return parmat*np.linalg.inv(transform @ R)
+    else:
+        return parmat*(transform @ R)
+
 # Apply the values of the given transformation parameters to the
 # given cairo context. Generally should be used AFTER
 # pushPhysicalCoords() is called.
