@@ -7,7 +7,10 @@ pi = math.pi
 tau = 2*pi
 oo = inf = float("inf")
 nan = float("nan")
-
+# Basic unit vectors for 3D animations
+ihat = np.array([1,0,0], dtype=float)
+jhat = np.array([0,1,0], dtype=float)
+khat = np.array([0,0,1], dtype=float)
 
 ### FUNCTIONS ###
 
@@ -26,6 +29,16 @@ def isbadnum(x):
 def isbadarray(x):
     return (np.any(np.isnan(x)) or np.any(np.isinf(x))).tolist()
 
+# isequal(a,b) does a == b, but works even if a and/or b is
+# a numpy array.
+def isequal(a, b):
+    if isinstance(a, np.ndarray) and isinstance(b, np.ndarray):
+        return np.array_equal(a,b)
+    elif isinstance(a, np.ndarray) != isinstance(b, np.ndarray):
+        return False
+    else:
+        return (a == b)
+
 # Conversion factors between degrees and radians.
 deg = tau/360
 rad = 1/deg
@@ -45,6 +58,13 @@ def sgn(x):
     else:
         return 0
 
+# Constrains x to being in the interval [low, high].
+# If x is above high, constrain() returns high.
+# If x is below low, constrain() returns low.
+# Equivalent to min(max(x,low), high)
+def constrain(x, low, high):
+    return min(max(x, low), high)
+
 # Computes the mean of a vector-like thing.
 def mean(a):
     if len(a) == 0:
@@ -57,6 +77,30 @@ def truncate(num, ndigits):
     decshift = 10**ndigits
     return int(num*decshift)/decshift
 
+def _rounder(x, roundfunc):
+    # Convert to np.array if needed
+    array = np.array(x) if not isinstance(x, np.ndarray) else x
+    array = np.sign(array)*roundfunc(np.abs(array))
+    array = np.array(array, dtype=int)  # Convert to int array
+    if isinstance(x, np.ndarray):
+        return array
+    else:
+        return array.tolist()
+
+# Rounds the input away from zero.
+# Supports lists and np.arrays as input.
+#   roundOut(1.1) --> 2
+#   roundOut(-1.1) --> -2
+def roundOut(x):
+    return _rounder(x, np.ceil)
+
+# Rounds the input toward zero.
+# Supports lists and np.arrays as input.
+#   roundIn(1.9) --> 1
+#   roundIn(-1.9) --> -1
+def roundIn(x):
+    return _rounder(x, np.floor)
+
 # If a float is equal to an int, converts it into an int.
 def squeezeFloat(number):
     try:
@@ -161,6 +205,11 @@ def arcCenterArray(p, q, angle):
 def vect2complex(v):
     return v[0] + 1j*v[1]
 
+# Converts a complex number into an ordered pair
+# (z.real, z.imag)
+def complex2vect(z):
+    return (z.real, z.imag)
+
 # Given a sorted list of numbers a and value x,
 # returns the highest index i such that a[i] <= x.
 # If all the numbers in a are larger than x, it returns -1.
@@ -233,3 +282,16 @@ def boxCorners(box, initCorner="NW", CCW=True):
     corners = corners[i:] + corners[:i]
 
     return corners
+
+# Converts minutes with seconds into just seconds.
+# minsec(m,s) --> 60*m + s
+#
+# If given only a single decimal number as input, it treats
+# it as (minutes).(seconds) and converts it to seconds.
+# Example: minsec(2.05) --> 125 (within rounding error)
+def minsec(mins, secs=None, /):
+    if secs is None:
+        value = mins
+        mins = int(value)
+        secs = (value-mins)/0.6 * 60
+    return 60*mins + secs
